<?php //$Id: restorelib.php,v 1.19 2005/07/12 08:00:38 moodler Exp $
    //This php script contains all the stuff to backup/restore
    //miechoice mods

    //This is the "graphical" structure of the miechoice mod:
    //
    //                      miechoice
    //                    (CL,pk->id)----------|
    //                        |                |
    //                        |                |
    //                        |                |
    //                  miechoice_options         |
    //             (UL,pk->id, fk->miechoiceid)   |
    //                        |                |
    //                        |                |
    //                        |                |
    //                   miechoice_answers        |
    //        (UL,pk->id, fk->miechoiceid, fk->optionid)       
    //
    // Meaning: pk->primary key field of the table
    //          fk->foreign key to link with parent
    //          nt->nested field (recursive data)
    //          CL->course level info
    //          UL->user level info
    //          files->table may have files)
    //
    //-----------------------------------------------------------

    //This function executes all the restore procedure about this mod
    function miechoice_restore_mods($mod,$restore) {

        global $CFG;

        $status = true;

        //Get record from backup_ids
        $data = backup_getid($restore->backup_unique_code,$mod->modtype,$mod->id);

        if ($data) {
            //Now get completed xmlized object
            $info = $data->info;
            //traverse_xmlize($info);                                                                     //Debug
            //print_object ($GLOBALS['traverse_array']);                                                  //Debug
            //$GLOBALS['traverse_array']="";                                                              //Debug

            //Now, build the MIECHOICE record structure
            $miechoice->course = $restore->course_id;
            $miechoice->groupid = backup_todb($info['MOD']['#']['GROUPID']['0']['#']);
            $miechoice->name = backup_todb($info['MOD']['#']['NAME']['0']['#']);
            $miechoice->text = backup_todb($info['MOD']['#']['TEXT']['0']['#']);
            $miechoice->format = backup_todb($info['MOD']['#']['FORMAT']['0']['#']);
            $miechoice->publish = backup_todb($info['MOD']['#']['PUBLISH']['0']['#']);
            $miechoice->rlsresult = backup_todb($info['MOD']['#']['RLSRESULT']['0']['#']);
            $miechoice->display = backup_todb($info['MOD']['#']['DISPLAY']['0']['#']);
            $miechoice->allowupdate = backup_todb($info['MOD']['#']['ALLOWUPDATE']['0']['#']);
            $miechoice->showunanswered = backup_todb($info['MOD']['#']['SHOWUNANSWERED']['0']['#']);
            $miechoice->limitanswers = backup_todb($info['MOD']['#']['LIMITANSWERS']['0']['#']);
            $miechoice->timeopen = backup_todb($info['MOD']['#']['TIMEOPEN']['0']['#']);
            $miechoice->timeclose = backup_todb($info['MOD']['#']['TIMECLOSE']['0']['#']);
            $miechoice->timemodified = backup_todb($info['MOD']['#']['TIMEMODIFIED']['0']['#']);

            //To mantain compatibilty, in 1.4 the publish setting meaning has changed. We
            //have to modify some things it if the release field isn't present in the backup file.
            if (! isset($info['MOD']['#']['RELEASE']['0']['#'])) {  //It's a pre-14 backup filea
                //Set the allowupdate field
                if ($miechoice->publish == 0) { 
                    $miechoice->allowupdate = 1;
                }
                //Set the release field as defined by the old publish field
                if ($miechoice->publish > 0) {
                    $miechoice->rlsresult = 1;
                }
                //Recode the publish field to its 1.4 meaning
                if ($miechoice->publish > 0) {
                    $miechoice->publish--;
                }
            }
            
            //The structure is equal to the db, so insert the miechoice
            $newid = insert_record ("miechoice",$miechoice);
            
            if ($newid) {
                //We have the newid, update backup_ids
                backup_putid($restore->backup_unique_code,$mod->modtype,
                             $mod->id, $newid);
                
                //Check to see how answers (curently miechoice_options) are stored in the table 
                //If answer1 - answer6 exist, this is a pre 1.5 version of miechoice
                if (isset($info['MOD']['#']['ANSWER1']['0']['#']) || 
                    isset($info['MOD']['#']['ANSWER2']['0']['#']) || 
                    isset($info['MOD']['#']['ANSWER3']['0']['#']) || 
                    isset($info['MOD']['#']['ANSWER4']['0']['#']) || 
                    isset($info['MOD']['#']['ANSWER5']['0']['#']) || 
                    isset($info['MOD']['#']['ANSWER6']['0']['#']) ) {
              
                    //This is a pre 1.5 miechoice backup, special work begins
                    $options = array();
                    $options[1] = backup_todb($info['MOD']['#']['ANSWER1']['0']['#']);
                    $options[2] = backup_todb($info['MOD']['#']['ANSWER2']['0']['#']);
                    $options[3] = backup_todb($info['MOD']['#']['ANSWER3']['0']['#']);
                    $options[4] = backup_todb($info['MOD']['#']['ANSWER4']['0']['#']);
                    $options[5] = backup_todb($info['MOD']['#']['ANSWER5']['0']['#']);
                    $options[6] = backup_todb($info['MOD']['#']['ANSWER6']['0']['#']);
                
                    for($i = 1; $i < 7; $i++) { //insert old answers (in 1.4)  as miechoice_options (1.5) to db.  
                        if (!empty($options[$i])) {  //make sure this option has something in it!
                            $option->miechoiceid = $newid;
                            $option->text = $options[$i];
                            $option->timemodified = $miechoice->timemodified;
                            $newoptionid = insert_record ("miechoice_options",$option);
                            //Save this miechoice_option to backup_ids
                            backup_putid($restore->backup_unique_code,"miechoice_options",$i,$newoptionid);
                        }
                    }
                 } else { //Now we are in a "standard" 1.5 miechoice, so restore miechoice_options normally
                     $status = miechoice_options_restore_mods($newid,$info,$restore);
                 }

                 //now restore the answers for this miechoice.
                 if ($restore->mods['miechoice']->userinfo) {
                    //Restore miechoice_answers
                    $status = miechoice_answers_restore_mods($newid,$info,$restore);     
                 }                               
            } else {
                $status = false;
            }

            //Do some output     
            echo "<li>".get_string("modulename","miechoice")." \"".format_string(stripslashes($miechoice->name),true)."\"</li>";
            backup_flush(300);

        } else {
            $status = false;
        }
        return $status;
    }

function miechoice_options_restore_mods($miechoiceid,$info,$restore) {

        global $CFG;

        $status = true;

        $options = $info['MOD']['#']['OPTIONS']['0']['#']['OPTION'];

        //Iterate over options
        for($i = 0; $i < sizeof($options); $i++) {
            $opt_info = $options[$i];
            //traverse_xmlize($opt_info);                                                                 //Debug
            //print_object ($GLOBALS['traverse_array']);                                                  //Debug
            //$GLOBALS['traverse_array']="";                                                              //Debug

            //We'll need this later!!
            $oldid = backup_todb($opt_info['#']['ID']['0']['#']);
            $olduserid = backup_todb($opt_info['#']['USERID']['0']['#']);

            //Now, build the MIECHOICE_OPTIONS record structure
            $option->miechoiceid = $miechoiceid;
            $option->text = backup_todb($opt_info['#']['TEXT']['0']['#']);
            $option->maxanswers = backup_todb($opt_info['#']['MAXANSWERS']['0']['#']);
            $option->timemodified = backup_todb($opt_info['#']['TIMEMODIFIED']['0']['#']);

            //The structure is equal to the db, so insert the miechoice_options
            $newid = insert_record ("miechoice_options",$option);

            //Do some output
            if (($i+1) % 50 == 0) {
                echo ".";
                if (($i+1) % 1000 == 0) {
                    echo "<br />";
                }
                backup_flush(300);
            }

            if ($newid) {
                //We have the newid, update backup_ids
                backup_putid($restore->backup_unique_code,"miechoice_options",$oldid,
                             $newid);
            } else {
                $status = false;
            }
        }

        return $status;
    }

    //This function restores the miechoice_answers
    function miechoice_answers_restore_mods($miechoiceid,$info,$restore) {

        global $CFG;

        $status = true;

        $answers = $info['MOD']['#']['ANSWERS']['0']['#']['ANSWER'];

        //Iterate over answers
        for($i = 0; $i < sizeof($answers); $i++) {
            $ans_info = $answers[$i];
            //traverse_xmlize($sub_info);                                                                 //Debug
            //print_object ($GLOBALS['traverse_array']);                                                  //Debug
            //$GLOBALS['traverse_array']="";                                                              //Debug

            //We'll need this later!!
            $oldid = backup_todb($ans_info['#']['ID']['0']['#']);
            $olduserid = backup_todb($ans_info['#']['USERID']['0']['#']);

            //Now, build the MIECHOICE_ANSWERS record structure
            $answer->miechoiceid = $miechoiceid;
            $answer->userid = backup_todb($ans_info['#']['USERID']['0']['#']);
            $answer->optionid = backup_todb($ans_info['#']['OPTIONID']['0']['#']);
            $answer->timemodified = backup_todb($ans_info['#']['TIMEMODIFIED']['0']['#']);

            //If the answer contains MIECHOICE_ANSWER, it's a pre 1.5 backup
            if (!empty($ans_info['#']['MIECHOICE_ANSWER']['0']['#'])) {
                //optionid was, in pre 1.5 backups, miechoice_answer
                $answer->optionid = backup_todb($ans_info['#']['MIECHOICE_ANSWER']['0']['#']);
            }

            //We have to recode the optionid field
            $option = backup_getid($restore->backup_unique_code,"miechoice_options",$answer->optionid);
            if ($option) {
                $answer->optionid = $option->new_id;
            }

            //We have to recode the userid field
            $user = backup_getid($restore->backup_unique_code,"user",$answer->userid);
            if ($user) {
                $answer->userid = $user->new_id;
            }

            //The structure is equal to the db, so insert the miechoice_answers
            $newid = insert_record ("miechoice_answers",$answer);

            //Do some output
            if (($i+1) % 50 == 0) {
                echo ".";
                if (($i+1) % 1000 == 0) {
                    echo "<br />";
                }
                backup_flush(300);
            }

            if ($newid) {
                //We have the newid, update backup_ids
                backup_putid($restore->backup_unique_code,"miechoice_answers",$oldid,
                             $newid);
            } else {
                $status = false;
            }
        }

        return $status;
    }

    //Return a content decoded to support interactivities linking. Every module
    //should have its own. They are called automatically from
    //miechoice_decode_content_links_caller() function in each module
    //in the restore process
    function miechoice_decode_content_links ($content,$restore) {
            
        global $CFG;
            
        $result = $content;
                
        //Link to the list of miechoices
                
        $searchstring='/\$@(MIECHOICEINDEX)\*([0-9]+)@\$/';
        //We look for it
        preg_match_all($searchstring,$content,$foundset);
        //If found, then we are going to look for its new id (in backup tables)
        if ($foundset[0]) {
            //print_object($foundset);                                     //Debug
            //Iterate over foundset[2]. They are the old_ids
            foreach($foundset[2] as $old_id) {
                //We get the needed variables here (course id)
                $rec = backup_getid($restore->backup_unique_code,"course",$old_id);
                //Personalize the searchstring
                $searchstring='/\$@(MIECHOICEINDEX)\*('.$old_id.')@\$/';
                //If it is a link to this course, update the link to its new location
                if($rec->new_id) {
                    //Now replace it
                    $result= preg_replace($searchstring,$CFG->wwwroot.'/mod/miechoice/index.php?id='.$rec->new_id,$result);
                } else { 
                    //It's a foreign link so leave it as original
                    $result= preg_replace($searchstring,$restore->original_wwwroot.'/mod/miechoice/index.php?id='.$old_id,$result);
                }
            }
        }

        //Link to miechoice view by moduleid

        $searchstring='/\$@(MIECHOICEVIEWBYID)\*([0-9]+)@\$/';
        //We look for it
        preg_match_all($searchstring,$result,$foundset);
        //If found, then we are going to look for its new id (in backup tables)
        if ($foundset[0]) {
            //print_object($foundset);                                     //Debug
            //Iterate over foundset[2]. They are the old_ids
            foreach($foundset[2] as $old_id) {
                //We get the needed variables here (course_modules id)
                $rec = backup_getid($restore->backup_unique_code,"course_modules",$old_id);
                //Personalize the searchstring
                $searchstring='/\$@(MIECHOICEVIEWBYID)\*('.$old_id.')@\$/';
                //If it is a link to this course, update the link to its new location
                if($rec->new_id) {
                    //Now replace it
                    $result= preg_replace($searchstring,$CFG->wwwroot.'/mod/miechoice/view.php?id='.$rec->new_id,$result);
                } else {
                    //It's a foreign link so leave it as original
                    $result= preg_replace($searchstring,$restore->original_wwwroot.'/mod/miechoice/view.php?id='.$old_id,$result);
                }
            }
        }

        return $result;
    }

    //This function makes all the necessary calls to xxxx_decode_content_links()
    //function in each module, passing them the desired contents to be decoded
    //from backup format to destination site/course in order to mantain inter-activities
    //working in the backup/restore process. It's called from restore_decode_content_links()
    //function in restore process
    function miechoice_decode_content_links_caller($restore) {
        global $CFG;
        $status = true;
        
        if ($miechoices = get_records_sql ("SELECT c.id, c.text
                                   FROM {$CFG->prefix}miechoice c
                                   WHERE c.course = $restore->course_id")) {
                                               //Iterate over each miechoice->text
            $i = 0;   //Counter to send some output to the browser to avoid timeouts
            foreach ($miechoices as $miechoice) {
                //Increment counter
                $i++;
                $content = $miechoice->text;
                $result = restore_decode_content_links_worker($content,$restore);
                if ($result != $content) {
                    //Update record
                    $miechoice->text = addslashes($result);
                    $status = update_record("miechoice",$miechoice);
                    if ($CFG->debug>7) {
                        echo '<br /><hr />'.htmlentities($content,ENT_COMPAT,get_string('thischarset')).'<br />changed to<br />'.htmlentities($result,ENT_COMPAT,get_string('thischarset')).'<hr /><br />';
                    }
                }
                //Do some output
                if (($i+1) % 5 == 0) {
                    echo ".";
                    if (($i+1) % 100 == 0) {
                        echo "<br />";
                    }
                    backup_flush(300);
                    }
            }
        }

        return $status;
    }

    //This function converts texts in FORMAT_WIKI to FORMAT_MARKDOWN for
    //some texts in the module
    function miechoice_restore_wiki2markdown ($restore) {
    
        global $CFG;

        $status = true;

        //Convert miechoice->text
        if ($records = get_records_sql ("SELECT c.id, c.text, c.format
                                         FROM {$CFG->prefix}miechoice c,
                                              {$CFG->prefix}backup_ids b
                                         WHERE c.course = $restore->course_id AND
                                               c.format = ".FORMAT_WIKI. " AND
                                               b.backup_code = $restore->backup_unique_code AND
                                               b.table_name = 'miechoice' AND
                                               b.new_id = c.id")) {
            foreach ($records as $record) {
                //Rebuild wiki links
                $record->text = restore_decode_wiki_content($record->text, $restore);
                //Convert to Markdown
                $wtm = new WikiToMarkdown();
                $record->text = $wtm->convert($record->text, $restore->course_id);
                $record->format = FORMAT_MARKDOWN;
                $status = update_record('miechoice', addslashes_object($record));
                //Do some output
                $i++;
                if (($i+1) % 1 == 0) {
                    echo ".";
                    if (($i+1) % 20 == 0) {
                        echo "<br />";
                    }
                    backup_flush(300);
                }
            }

        }
        return $status;
    }

    //This function returns a log record with all the necessay transformations
    //done. It's used by restore_log_module() to restore modules log.
    function miechoice_restore_logs($restore,$log) {
                    
        $status = false;
                    
        //Depending of the action, we recode different things
        switch ($log->action) {
        case "add":
            if ($log->cmid) {
                //Get the new_id of the module (to recode the info field)
                $mod = backup_getid($restore->backup_unique_code,$log->module,$log->info);
                if ($mod) {
                    $log->url = "view.php?id=".$log->cmid;
                    $log->info = $mod->new_id;
                    $status = true;
                }
            }
            break;
        case "update":
            if ($log->cmid) {
                //Get the new_id of the module (to recode the info field)
                $mod = backup_getid($restore->backup_unique_code,$log->module,$log->info);
                if ($mod) {
                    $log->url = "view.php?id=".$log->cmid;
                    $log->info = $mod->new_id;
                    $status = true;
                }
            }
            break;
        case "choose":
            if ($log->cmid) {
                //Get the new_id of the module (to recode the info field)
                $mod = backup_getid($restore->backup_unique_code,$log->module,$log->info);
                if ($mod) {
                    $log->url = "view.php?id=".$log->cmid;
                    $log->info = $mod->new_id;
                    $status = true;
                }
            }
            break;
        case "choose again":
            if ($log->cmid) {
                //Get the new_id of the module (to recode the info field)
                $mod = backup_getid($restore->backup_unique_code,$log->module,$log->info);
                if ($mod) {
                    $log->url = "view.php?id=".$log->cmid;
                    $log->info = $mod->new_id;
                    $status = true;
                }
            }
            break;
        case "view":
            if ($log->cmid) {
                //Get the new_id of the module (to recode the info field)
                $mod = backup_getid($restore->backup_unique_code,$log->module,$log->info);
                if ($mod) {
                    $log->url = "view.php?id=".$log->cmid;
                    $log->info = $mod->new_id;
                    $status = true;
                }
            }
            break;
        case "view all":
            $log->url = "index.php?id=".$log->course;
            $status = true;
            break;
        case "report":
            if ($log->cmid) {
                //Get the new_id of the module (to recode the info field)
                $mod = backup_getid($restore->backup_unique_code,$log->module,$log->info);
                if ($mod) {
                    $log->url = "report.php?id=".$log->cmid;
                    $log->info = $mod->new_id;
                    $status = true;
                }
            }
            break;
        default:
            echo "action (".$log->module."-".$log->action.") unknow. Not restored<br />";                 //Debug
            break;
        }

        if ($status) {
            $status = $log;
        }
        return $status;
    }
?>
