<?php // $Id: questiontypes.class.php,v 1.20 2007/01/08 15:27:05 thepurpleblob Exp $

/**
 * This file contains the parent class for questionnaire question types.
 *
 * @author Mike Churchward
 * @version  
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package questiontypes
 */

/**
 * Class for describing a question
 *
 * @author Mike Churchward
 * @package questiontypes
 */
 
 /// Constants
define('QUESYESNO', 1);
define('QUESTEXT', 2);
define('QUESESSAY', 3);
define('QUESRADIO', 4);
define('QUESCHECK', 5);
define('QUESDROP', 6);
define('QUESRATE', 8);
define('QUESDATE', 9);
define('QUESNUMERIC', 10);
define('QUESPAGEBREAK', 99);
define('QUESSECTIONTEXT', 100);
 
$QTYPENAMES = array(
        QUESYESNO =>        'yesno',
        QUESTEXT =>         'text',
        QUESESSAY    =>     'essay',
        QUESRADIO =>        'radio',
        QUESCHECK =>        'check',
        QUESDROP =>         'drop',
        QUESRATE =>         'rate',
        QUESDATE =>         'date',
        QUESNUMERIC =>      'numeric',
        QUESPAGEBREAK =>    'pagebreak',
        QUESSECTIONTEXT =>  'sectiontext'
        );
 
class questionnaire_question {

/// Class Properties
    /**
     * The database id of this question.
     * @var int $id
     */
     var $id    = 0;

    /**
     * The database id of the survey this question belongs to.
     * @var int $survey_id
     */
     var $survey_id    = 0;

    /**
     * The name of this question.
     * @var string $name
     */
     var $name    = '';

    /**
     * The name of the question type.
     * @var string $type
     */
     var $type    = '';

    /**
     * Array holding any choices for this question.
     * @var array $choices
     */
     var $choices    = array();

    /**
     * The table name for responses.
     * @var string $response_table
     */
     var $response_table    = '';

    /**
     * The length field.
     * @var int $length
     */
     var $length    = 0;

    /**
     * The precision field.
     * @var int $precise
     */
     var $precise    = 0;

    /**
     * Position in the questionnaire
     * @var int $position
     */
     var $position    = 0;

    /**
     * The question's content.
     * @var string $content
     */
     var $content    = '';

    /**
     * The required flag.
     * @var boolean $required
     */
     var $required    = 'N';

    /**
     * The deleted flag.
     * @var boolean $deleted
     */
     var $deleted    = 'N';

    /**
     * The public flag.
     * @var boolean $public
     */
     var $public    = 'N';

/// Class Methods

    /**
     * The class constructor
     *
     */
    function questionnaire_question($id = 0, $question = null) {

        static $qtypes = null;

        if (is_null($qtypes)) {
            $qtypes = get_records('questionnaire_question_type');
        }

        if ($id) {
            $question = get_record('questionnaire_question', 'id', $id);
        } 

        if (is_object($question)) {
            $this->id = $question->id;
            $this->survey_id = $question->survey_id;
            $this->name = $question->name;
            $this->length = $question->length;
            $this->precise = $question->precise;
            $this->position = $question->position;
            $this->content = $question->content;
            $this->required = $question->required;
            $this->deleted = $question->deleted;
            $this->public = $question->public;

            $this->type_id = $question->type_id;
            $this->type = $qtypes[$this->type_id]->type;
            $this->response_table = $qtypes[$this->type_id]->response_table;
            if ($qtypes[$this->type_id]->has_choices == 'Y') {
                $this->get_choices();
            }
        }
    }

    /**
     * Fake constructor to keep PHP5 happy
     *
     */
    function __construct($id = 0, $question = null) {
        $this->questionnaire_question($id, $question);
    }

    function get_choices() {
        if ($choices = get_records('questionnaire_question_choice', 'question_id', $this->id, 'id ASC')) {
            foreach ($choices as $choice) {
                $this->choices[$choice->id]->content = $choice->content;
                $this->choices[$choice->id]->value = $choice->value;
            }
        } else {
            $this->choices = array();
        }
    }

/// Storage Methods:
/// The following methods are defined by the tables they use. Questions should call the 
/// appropriate function based on its table.

    function insert_response($rid, $formdata) {
        if (isset($formdata->{'q'.$this->id})) {
            $val = clean_param($formdata->{'q'.$this->id}, PARAM_CLEAN);
        }
        else {
            $val = '';
        }

        $method = 'insert_'.$this->response_table;
        if (method_exists($this, $method)) {
            return $this->$method($rid, $formdata);
        } else {
            return false;
        }
    }

    function insert_response_bool($rid, $formdata) {
        if (isset($formdata->{'q'.$this->id})) {
            $record = new Object();
            $record->response_id = $rid;
            $record->question_id = $this->id;
            $record->choice_id = $formdata->{'q'.$this->id};
            return insert_record('questionnaire_'.$this->response_table, $record);
        } else {
            return false;
        }
    }

    function insert_response_text($rid, $formdata) {
        // only insert if non-empty content
        if($this->type_id == 10) { // numeric
            $formdata->{'q'.$this->id} = ereg_replace("[^0-9.\-]*(-?[0-9]*\.?[0-9]*).*", '\1', $formdata->{'q'.$this->id});
        }

        if(ereg("[^ \t\n]",$formdata->{'q'.$this->id})) {
            $record = new Object();
            $record->response_id = $rid;
            $record->question_id = $this->id;
            $record->response = $formdata->{'q'.$this->id};
            return insert_record('questionnaire_'.$this->response_table, $record);
        } else {
            return false;
        }
    }

    function insert_response_date($rid, $formdata) {
        // only insert if non-empty content
        /// Use configured date format:
        $dateformat = trim(QUESTIONNAIREDATEFORMAT);
        if (preg_match('/(%[mdyY])(.+)(%[mdyY])(.+)(%[mdyY])/', $dateformat, $matches)) {
            $date_pieces = explode($matches[2], $formdata->{'q'.$this->id});
            if (count($date_pieces) > 1) {
                if ($matches[1] == '%m') $month = $date_pieces[0];
                if ($matches[1] == '%d') $day = $date_pieces[0];
                if ($matches[1] == '%y') $year = strftime('%C').$date_pieces[0];
                if ($matches[1] == '%Y') $year = $date_pieces[0];

                if ($matches[2] != $matches[4]) {
                    $date_pieces = explode($date_pieces[1], $matches[4]);
                    $date_pieces[2] = $date_pieces[1]; $date_pieces[1] = $date_pieces[0];
                }

                if ($matches[3] == '%m') $month = $date_pieces[1];
                if ($matches[3] == '%d') $day = $date_pieces[1];
                if ($matches[3] == '%y') $year = strftime('%C').$date_pieces[1];
                if ($matches[3] == '%Y') $year = $date_pieces[1];

                if ($matches[5] == '%m') $month = $date_pieces[2];
                if ($matches[5] == '%d') $day = $date_pieces[2];
                if ($matches[5] == '%y') $year = strftime('%C').$date_pieces[2];
                if ($matches[5] == '%Y') $year = $date_pieces[2];

                $month = min(12,$month);
                $month = max(1,$month);
                if ($month == 2) {
                    $day = min(29, $day);
                } else if ($month == 4 || $month == 6 || $month == 9 || $month == 11) {    
                    $day = min(30, $day);
                } else {
                    $day = min(31, $day);
                }
                $day = max(1, $day);
                $val = addslashes($year.'-'.$month.'-'.$day);
                $record = new Object();
                $record->response_id = $rid;
                $record->question_id = $this->id;
                $record->response = $val;
                return insert_record('questionnaire_'.$this->response_table, $record);
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    
    function insert_response_single($rid, $formdata) {    

        if(!empty($formdata->{'q'.$this->id})) {
            foreach ($this->choices as $cid => $choice) {
                if (strpos($choice->content, '!other') === 0) {
                    if (!isset($formdata->{'q'.$this->id.'_'.$cid})) {
                        continue;
                    }
                    $other = clean_param($formdata->{'q'.$this->id.'_'.$cid}, PARAM_CLEAN);
                    if(ereg("[^ \t\n]",$other)) {
                        $record = new Object();
                        $record->response_id = $rid;
                        $record->question_id = $this->id;
                        $record->choice_id = $cid;
                        $record->response = $other;
                        $resid = insert_record('questionnaire_response_other', $record);
                        $formdata->{'q'.$this->id} = $cid;
                        break;
                    }
                }
            }
        }
        if(ereg("other_q([0-9]+)", (isset($formdata->{'q'.$this->id})?$formdata->{'q'.$this->id}:''), $regs)) {
            $cid=$regs[1];
            if (!isset($formdata->{'q'.$this->id.'_'.$cid})) {
                break; // out of the case
            }
            $other = clean_param($formdata->{'q'.$this->id.'_'.$cid}, PARAM_CLEAN);
            if(ereg("[^ \t\n]",$other)) {
                $record = new object;
                $record->response_id = $rid;
                $record->question_id = $this->id;
                $record->choice_id = $cid;
                $record->response = $other;
                $resid = insert_record('questionnaire_response_other', $record);
                $formdata->{'q'.$this->id} = $cid;
            }
        }
        $record = new Object();
        $record->response_id = $rid;
        $record->question_id = $this->id;
        $record->choice_id = isset($formdata->{'q'.$this->id}) ? $formdata->{'q'.$this->id} : 0;
        return insert_record('questionnaire_'.$this->response_table, $record);
    }
    
    function insert_response_multiple($rid, $formdata) {

        foreach ($this->choices as $cid => $choice) {
            if (strpos($choice->content, '!other') === 0) {
                if (!isset($formdata->{'q'.$this->id.'_'.$cid}) || empty($formdata->{'q'.$this->id.'_'.$cid})) {
                    continue;
                }
                if (!isset($formdata->{'q'.$this->id})) {
                    $formdata->{'q'.$this->id} = array($cid);
                } else {
                    array_push($formdata->{'q'.$this->id}, $cid);
                }
                $other = clean_param($formdata->{'q'.$this->id.'_'.$cid}, PARAM_CLEAN);
                if(ereg("[^ \t\n]",$other)) {
                    $record = new Object();
                    $record->response_id = $rid;
                    $record->question_id = $this->id;
                    $record->choice_id = $cid;
                    $record->response = $other;
                    $resid = insert_record('questionnaire_response_other', $record);
                }
            }
        }

        if(!isset($formdata->{'q'.$this->id}) || count($formdata->{'q'.$this->id}) < 1) {
            return false;
        }

        foreach($formdata->{'q'.$this->id} as $cid) {
            $cid = clean_param($cid, PARAM_CLEAN);
            if(ereg("other_q[0-9]+", $cid))
                continue;
            $record = new Object();
            $record->response_id = $rid;
            $record->question_id = $this->id;
            $record->choice_id = $cid;
            $resid = insert_record('questionnaire_'.$this->response_table, $record);
        }
        return $resid;
    }
    
    function insert_response_rank($rid, $formdata) {
        if($this->type_id == 8) { // Rank
            $resid = false;
            foreach ($this->choices as $cid => $choice) {
                if (!isset($formdata->{'q'.$this->id.'_'.$cid})) {
                    continue;
                }
                $val = clean_param($formdata->{'q'.$this->id.'_'.$cid}, PARAM_CLEAN);
                if(strtolower($val) == "n/a") {
                    $rank = -1;
                } else {
                    $rank = intval($val);
                }
                $record = new Object();
                $record->response_id = $rid;
                $record->question_id = $this->id;
                $record->choice_id = $cid;
                $record->rank = $rank;
                $resid = insert_record('questionnaire_'.$this->response_table, $record);
            }
            return $resid;
        } else {
            if(strtolower($formdata->{'q'.$this->id}) == "n/a") {
                $rank = -1;
            } else {
                $rank = intval($formdata->{'q'.$this->id});
            }
            $record = new Object();
            $record->response_id = $rid;
            $record->question_id = $this->id;
            $record->rank = $rank;
            return insert_record('questionnaire_'.$this->response_table, $record);
        }
    }


/// Results Methods:
/// The following methods are defined by the tables they use. Questions should call the 
/// appropriate function based on its table.

    function get_results($rids=false) {

        $method = 'get_'.$this->response_table.'_results';
        if (method_exists($this, $method)) {
            return $this->$method($rids);
        } else {
            return false;
        }
    }

    function get_response_bool_results($rids=false) {
        global $CFG;

        $ridstr = '';
        if (is_array($rids)) {
            foreach ($rids as $rid) {
                $ridstr .= (empty($ridstr) ? ' AND response_id IN ('.$rid : ', '.$rid);
            }
            $ridstr .= ') ';
        } else if (is_int($rids)) {
            $ridstr = ' AND response_id = '.$rids.' ';
        }

        $sql = 'SELECT choice_id, COUNT(response_id) AS num '.
               'FROM '.$CFG->prefix.'questionnaire_'.$this->response_table.' '.
               'WHERE question_id='.$this->id.$ridstr.' AND choice_id != \'\' '.
               'GROUP BY choice_id';

        return get_records_sql($sql);
    }

    function get_response_text_results($rids = false) {
        global $CFG;

        $ridstr = '';
        if (is_array($rids)) {
            foreach ($rids as $rid) {
                $ridstr .= (empty($ridstr) ? ' AND response_id IN ('.$rid : ', '.$rid);
            }
            $ridstr .= ') ';
        } else if (is_int($rids)) {
            $ridstr = ' AND response_id = '.$rids.' ';
        }

        $sql = 'SELECT response, COUNT(response_id) AS num '.
               'FROM '.$CFG->prefix.'questionnaire_'.$this->response_table.' '.
               'WHERE question_id='.$this->id.$ridstr.' '.
               'GROUP BY response';

        return get_records_sql($sql);
    }

    function get_response_date_results($rids = false) {
        global $CFG;

        $ridstr = '';
        if (is_array($rids)) {
            foreach ($rids as $rid) {
                $ridstr .= (empty($ridstr) ? ' AND response_id IN ('.$rid : ', '.$rid);
            }
            $ridstr .= ') ';
        } else if (is_int($rids)) {
            $ridstr = ' AND response_id = '.$rids.' ';
        }

        $sql = 'SELECT response, COUNT(response_id) AS num '.
               'FROM '.$CFG->prefix.'questionnaire_'.$this->response_table.' '.
               'WHERE question_id='.$this->id.$ridstr.' '.
               'GROUP BY response';

        return get_records_sql($sql);
    }

    function get_response_single_results($rids=false) {
        global $CFG;

        $ridstr = '';
        if (is_array($rids)) {
            foreach ($rids as $rid) {
                $ridstr .= (empty($ridstr) ? ' AND response_id IN ('.$rid : ', '.$rid);
            }
            $ridstr .= ') ';
        } else if (is_int($rids)) {
            $ridstr = ' AND response_id = '.$rids.' ';
        }

//        $select = 'question_id='.$this->id.' AND content NOT LIKE \'!other%\'';
//        if ($rows = get_records_select('questionnaire_question_choice', $select, 'id', 'id,content')) {
//            foreach ($rows as $row) {
//                $this->counts[$row->content] = 0;
//            }
//        }

        $sql = 'SELECT qc.content, COUNT(rt.response_id) AS num '.
               'FROM '.$CFG->prefix.'questionnaire_question_choice qc, '.
                       $CFG->prefix.'questionnaire_'.$this->response_table.' rt '.
               'WHERE qc.question_id='.$this->id.' AND qc.content NOT LIKE \'!other%\' AND '.
                     'rt.question_id=qc.question_id AND rt.choice_id=qc.id'.$ridstr.' '.
               'GROUP BY qc.content';

        $rows = get_records_sql($sql);

        // handle 'other...'
        $sql = 'SELECT rt.response, qc.content '.
               'FROM '.$CFG->prefix.'questionnaire_response_other rt, '.
                       $CFG->prefix.'questionnaire_question_choice qc '.
               'WHERE rt.question_id='.$this->id.' AND rt.choice_id=qc.id'.$ridstr.' '.
               'ORDER BY qc.id, rt.response';
        if ($recs = get_records_sql($sql)) {
        	$i = 1;
            foreach ($recs as $rec) {
            	$rows['other'.$i] = $rec;
                $i++;
            }
        }

        return $rows;
    }    

    function get_response_multiple_results($rids=false) {
        global $CFG;

        $ridstr = '';
        if (is_array($rids)) {
            foreach ($rids as $rid) {
                $ridstr .= (empty($ridstr) ? ' AND response_id IN ('.$rid : ', '.$rid);
            }
            $ridstr .= ') ';
        } else if (is_int($rids)) {
            $ridstr = ' AND response_id = '.$rids.' ';
        }

//        $select = 'question_id='.$this->id.' AND content NOT LIKE \'!other%\'';
//        if ($rows = get_records_select('questionnaire_question_choice', $select, 'id', 'id,content')) {
//            foreach ($rows as $row) {
//                $this->counts[$row->content] = 0;
//            }
//        }

        $sql = 'SELECT qc.content, COUNT(rt.response_id) AS num '.
               'FROM '.$CFG->prefix.'questionnaire_question_choice qc, '.
                       $CFG->prefix.'questionnaire_'.$this->response_table.' rt '.
               'WHERE qc.question_id='.$this->id.' AND qc.content NOT LIKE \'!other%\' AND '.
                     'rt.question_id=qc.question_id AND rt.choice_id=qc.id'.$ridstr.' '.
               'GROUP BY qc.content';
        $rows = get_records_sql($sql);

        // handle 'other...'
        $sql = 'SELECT rt.response, qc.content '.
               'FROM '.$CFG->prefix.'questionnaire_response_other rt, '.
                       $CFG->prefix.'questionnaire_question_choice qc '.
               'WHERE rt.question_id='.$this->id.' AND rt.choice_id=qc.id'.$ridstr.' '.
               'ORDER BY qc.id, rt.response';
        if ($recs = get_records_sql($sql)) {
            $i = 1;
            foreach ($recs as $rec) {
                $rows['other'.$i] = $rec;
                $i++;
            }
        }

        return $rows;
    }

    function get_response_rank_results($rids=false) {
        global $CFG;

        $ridstr = '';
        if (is_array($rids)) {
            foreach ($rids as $rid) {
                $ridstr .= (empty($ridstr) ? ' AND response_id IN ('.$rid : ', '.$rid);
            }
            $ridstr .= ') ';
        } else if (is_int($rids)) {
            $ridstr = ' AND response_id = '.$rids.' ';
        }

        if($this->type_id  == 8) { //Rank
            $select = 'question_id='.$this->id.' AND content NOT LIKE \'!other%\'';
            if ($rows = get_records_select('questionnaire_question_choice', $select)) {
                foreach ($rows as $row) {
                    $this->counts[$row->content] = null;
                }
            }
            $sql = 'SELECT C.content, AVG(A.rank+1) AS average '.
                   'FROM '.$CFG->prefix.'questionnaire_question_choice C, '.
                           $CFG->prefix.'questionnaire_'.$this->response_table.' A '.
                   'WHERE C.question_id = '.$this->id.' AND A.question_id = '.$this->id.' AND '.
                         'A.choice_id = C.id AND A.rank >= 0'.$ridstr.' '.
                   'GROUP BY C.content';
            return get_records_sql($sql);
        } else {
            $sql = 'SELECT A.rank, COUNT(A.response_id) AS num '.
                   'FROM '.$CFG->prefix.'questionnaire_'.$this->response_table.' A '.
                   'WHERE A.question_id='.$this->id.$ridstr.' '.
                   'GROUP BY A.rank';
            return get_records_sql($sql);
        }
    }

/// Display Methods

    function display_results($rids=false) {

        $method = 'display_'.$this->response_table.'_results';
        if (method_exists($this, $method)) {
            return $this->$method($rids);
        } else {
            return false;
        }
    }

    function display_response_bool_results($rids=false) {
        if (empty($this->stryes)) {
            $this->stryes = get_string('yes');
            $this->strno = get_string('no');
        }
    
        if (is_array($rids)) {
            $prtotal = 1;
        } else if (is_int($rids)) {
            $prtotal = 0;
        }

        $this->counts = array($this->stryes => 0, $this->strno => 0);
        if ($rows = $this->get_response_bool_results($rids)) {
            foreach ($rows as $row) {
                $this->choice = $row->choice_id;
                $count = $row->num;
                if ($this->choice == 'Y') {
                    $this->choice = $this->stryes;
                } else {
                    $this->choice = $this->strno;
                }
                $this->counts[$this->choice] = $count;
            }
        }

        echo clean_text($this->content);
        $this->mkrespercent(count($rids), 1, $prtotal);
    }

    function display_response_text_results($rids = false) {
        if (is_array($rids)) {
            $prtotal = 1;
        } else if (is_int($rids)) {
            $prtotal = 0;
        }

        if ($rows = $this->get_response_text_results($rids)) {
            foreach ($rows as $row) {
                $this->text = $row->response;
                $count = $row->num;
                if(!empty($this->text)) {
                    $this->counts[clean_text($this->text)] = $count;
                }
            }
        }

        echo clean_text($this->content);
        $this->mkreslist(count($rids), 1, $prtotal);
    }

    function display_response_date_results($rids = false) {
        if (is_array($rids)) {
            $prtotal = 1;
        } else if (is_int($rids)) {
            $prtotal = 0;
        }

        if ($rows = $this->get_response_date_results($rids)) {
            foreach ($rows as $row) {
                $this->text = $row->response;
                $count = $row->num;
                if(!empty($this->text)) {
                    $dateparts = split('-', $this->text);
                    $this->text = strftime(QUESTIONNAIREDATEFORMAT,
                                           mktime(0, 0, 0, $dateparts[1], $dateparts[2], $dateparts[0]));
                    $this->counts[clean_text($this->text)] = $count;
                }
            }
        }

        echo clean_text($this->content);
        $this->mkreslist(count($rids), 1, $prtotal);
    }

    function display_response_single_results($rids=false) {
        if (is_array($rids)) {
            $prtotal = 1;
        } else if (is_int($rids)) {
            $prtotal = 0;
        }

        if ($rows = $this->get_response_single_results($rids)) {
            foreach ($rows as $idx => $row) {
                if (strpos($idx, 'other') === 0) {
                    $answer = $row->response;
                    $ccontent = $row->content;
                    $content = preg_replace(array('/^!other=/', '/^!other/'),
                            array('', 'Other:'), $ccontent);
                    $content .= ' ' . clean_text($answer);
                    $this->counts[$content] = (isset($this->counts[$content]) ? $this->counts[$content] : 0) + 1;
                } else {
                    $this->choice = $row->content;
                    $count = $row->num;
                    $this->counts[$this->choice] = $count;
                }
            }
        }

        echo clean_text($this->content);
        $this->mkrespercent(count($rids), 1, $prtotal);
    }    

    function display_response_multiple_results($rids=false) {
        if (is_array($rids)) {
            $prtotal = 1;
        } else if (is_int($rids)) {
            $prtotal = 0;
        }

        if ($rows = $this->get_response_multiple_results($rids)) {
            foreach ($rows as $idx => $row) {
                if (strpos($idx, 'other') === 0) {
                    $answer = $row->response;
                    $ccontent = $row->content;
                    $content = preg_replace(array('/^!other=/', '/^!other/'),
                            array('', 'Other:'), $ccontent);
                    $content .= ' ' . clean_text($answer);
                    $this->counts[$content] = (isset($this->counts[$content]) ? $this->counts[$content] : 0) + 1;
                } else {
                    $this->choice = $row->content;
                    $count = $row->num;
                    $this->counts[$this->choice] = $count;
                }
            }
        }

        echo clean_text($this->content);
        $this->mkrespercent(count($rids), 1, 0);
    }

    function display_response_rank_results($rids=false) {
        if (is_array($rids)) {
            $prtotal = 1;
        } else if (is_int($rids)) {
            $prtotal = 0;
        }

        if ($rows = $this->get_response_rank_results($rids)) {
            if($this->type_id  == 8) { //Rank
                foreach ($rows as $row) {
                    $ccontent = $row->content;
                    $avg = $row->average;
                    $this->counts[$ccontent] = $avg;
                }
                echo clean_text($this->content);
                $this->mkresavg(count($rids), 1, $prtotal, $this->length);
            } else {
                foreach ($rows as $row) {
                    $rank = $row->rank;
                    $num = $row->num;
                    if($rank == -1) { $rank = "N/A"; }
                    $this->counts[$rank] += $num;
                }
                echo clean_text($this->content);
                $this->mkresrank(count($rids), 1, $prtotal);
            }
        }
    }

    function question_display($data, $qnum='') {
        global $QTYPENAMES;

        $method = $QTYPENAMES[$this->type_id].'_survey_display';
        if (method_exists($this, $method)) {
            $this->questionstart_survey_display($qnum);
            $this->$method($data);
            $this->questionend_survey_display($qnum);
        } else {
            error('Display method not defined for question.');
        }
    }

    function survey_display($data, $qnum='', $usehtmleditor=null) {

        if (is_null($usehtmleditor)) {
            $this->usehtmleditor = can_use_html_editor();
        } else {
            $this->usehtmleditor = $usehtmleditor;
        }

        $this->question_display($data, $qnum);
    }

    function questionstart_survey_display($qnum) {
        if ($this->type_id == QUESSECTIONTEXT) {
            return;
        }

        echo '
    <div class="questioncontainer">
        <div class="qnOuter">
            <table class="qnInnerTable" width="100%" cellpadding="10"  cellspacing="1">
                <tr>
                    <td class="qnInnerTd" rowspan="2" valign="top">'.
                    ($this->required == 'Y' ? '<font color="#FF0000">*</font>':'').
                    '<a name="Q'.$qnum.'">'.$qnum.'.</a>
                    </td>
                    <td class="qnInner">'.
                    format_text($this->content, FORMAT_HTML).'
                    </td>
                </tr>
                <tr>
                    <td class="qnType">
        ';
    }

    function questionend_survey_display() {
        if ($this->type_id == QUESSECTIONTEXT) {
            return;
        }
        echo '
                    </td>
                </tr>
            </table>
         </div>
    </div>
        ';
    }

    function yesno_survey_display($data) {

        static $stryes = null;
        static $strno = null;

        if (is_null($stryes)) {
             $stryes = get_string('yes');
             $strno = get_string('no');
        }

    	$val1 = 'Y';
        $val2 = 'N';

        choose_from_radio(array($val1 => $stryes, $val2 => $strno), 'q'.$this->id, 
                          (isset($data->{'q'.$this->id})?$data->{'q'.$this->id}:''));
    }

    function text_survey_display($data) {
        echo '<input type="text" size="'.$this->length.'" name="q'.$this->id.'"'.
             ($this->precise > 0 ? ' maxlength="'.$this->precise.'"':'').' value="'.
             (isset($data->{'q'.$this->id})?$data->{'q'.$this->id}:'').'" />';
    }

    function essay_survey_display($data) {
        print_textarea($this->usehtmleditor, $this->precise, $this->length, 680, 400, 
                       'q'.$this->id, (isset($data->{'q'.$this->id})?$data->{'q'.$this->id}:''));
    }

    function radio_survey_display($data) {

        $idcounter = 0;
        $currentradio = 0;
        $checked = (isset($data->{'q'.$this->id})?$data->{'q'.$this->id}:'');
        $output = '<span class="radiogroup '.$this->id."\">\n";
        foreach ($this->choices as $id => $choice) {
            if (strpos($choice->content, '!other') !== 0) {
                $htmlid = 'auto-rb'.sprintf('%04d', ++$idcounter);
                $output .= ' <span class="radioelement q'.$this->id.' rb'.$currentradio."\">";
                $output .= '<input name="q'.$this->id.'" id="'.$htmlid.'" type="radio" value="'.$id.'"';
                if ($id == $checked) {
                    $output .= ' checked="1"';
                }
                if ($choice->content === '') {
                    $output .= ' /> <label for="'.$htmlid.'">'.$id.'</label></span>' .  "\n";
                } else {
                    $output .= ' /> <label for="'.$htmlid.'">'.format_text($choice->content, FORMAT_HTML).'</label></span>' .  "\n";
                }
                $currentradio = ($currentradio + 1) % 2;

            } else {
                $other_text = preg_replace(
                        array("/^!other=/","/^!other/"),
                        array('','Other:'),
                        $choice->content);
                $cid = 'q'.$this->id.'_'.$id;

                $htmlid = 'auto-rb'.sprintf('%04d', ++$idcounter);
                $output .= ' <span class="radioelement '.$this->id.' rb'.$currentradio."\">";
                $output .= '<input name="q'.$this->id.'" id="'.$htmlid.'" type="radio" value="other_'.$id.'"';
                if ('other_'.$id == $checked) {
                    $output .= ' checked="1"';
                }
                $output .= ' /> <label for="'.$htmlid.'">'.format_text($other_text, FORMAT_HTML).'</label></span>' .  "\n";
                $currentradio = ($currentradio + 1) % 2;

                $choices['other_'.$cid] = $other_text;
                $other_text = '&nbsp;<input type="text" size="20" name="'.$cid.'" onKeyPress="other_check(this.name)"';
                if (isset($data->$cid)) {
                    $other_text .= ' value="'. htmlspecialchars($data->$cid) .'"';
                }
                $other_text .= ' />';
                $output .= $other_text.' ';
            }
            $output .= '<br />';
        }
        $output .= '</span>' .  "\n";

        echo $output;
    }

    function check_survey_display($data) {
        if (!isset($data->{'q'.$this->id}) || !is_array($data->{'q'.$this->id})) {
            $data->{'q'.$this->id} = array();
        }

        foreach ($this->choices as $id => $choice) {
            if (strpos($choice->content, '!other') !== 0) {
                print_checkbox ('q'.$this->id.'[]', $id, in_array($id, $data->{'q'.$this->id}), 
                                format_text($choice->content, FORMAT_HTML));
                echo '<br />'."\n";
            } else {
                $other_text = preg_replace(
                        array("/^!other=/","/^!other/"),
                        array('','Other:'),
                        $choice->content);
                $cid = 'q'.$this->id.'_'.$id;
//                $other_text .= '&nbsp;<input type="text" size="20" name="'.$cid.'" onKeyPress="other_check(this.name)"';
//                if (isset($data->$cid)) {
//                    $other_text .= ' value="'. htmlspecialchars($data->$cid) .'"';
//                }
//                $other_text .= ' />';
                print_checkbox ('q'.$this->id.'[]', 'other_'.$id, isset($data->$cid), format_text($other_text, FORMAT_HTML));
                $other_text = '&nbsp;<input type="text" size="20" name="'.$cid.'" onKeyPress="other_check(this.name)"';
                if (isset($data->$cid)) {
                    $other_text .= ' value="'. htmlspecialchars($data->$cid) .'"';
                }
                $other_text .= ' />';
                echo $other_text.'<br />'."\n";
            }
        }
    }

    function drop_survey_display($data) {
        $options = array();
        foreach ($this->choices as $id => $choice) {
            $options[$id] = $choice->content;
        }
        choose_from_menu ($options, 'q'.$this->id, (isset($data->{'q'.$this->id})?$data->{'q'.$this->id}:''));
    }

    function rate_survey_display($data) {
    
        if (!isset($data->{'q'.$this->id}) || !is_array($data->{'q'.$this->id})) {
            $data->{'q'.$this->id} = array();
        }

        echo '<table border="0" cellspacing="1" cellpadding="0">'."\n";
        echo '  <tbody>'."\n";
        echo '    <tr>'."\n";
        echo '      <td></td>'."\n";
        $bg = '#eeeeee';
        for ($j = 0; $j < $this->length; $j++) {
            echo '      <td width="40" align="center" bgcolor="'.$bg.'">'.($j+1).'</td>'."\n";
            if ($bg == '#eeeeee') {
                $bg = '#dddddd';
            } else {
                $bg = '#eeeeee';
            }
        }
        if ($this->precise) {
            echo '      <td width="40" align="center" bgcolor="'.$bg.'">N/A</td>'."\n";
        }
        echo '    </tr>'."\n";
    
        foreach ($this->choices as $cid => $choice) {
            $str = 'q'."{$this->id}_$cid";
            echo '    <tr>'."\n";
            echo '      <td align="right">'.$choice->content.'</td>'."\n";
            $bg = '#eeeeee';
            for ($j = 0; $j < $this->length; $j++) {
                $checked = ((isset($data->$str) && ($j == $data->$str)) ? ' checked="checked"' : '');
                echo '      <td width="40" align="center" bgcolor="'.$bg.'">';
                echo '<input name="'.$str.'" type="radio" value="'.$j.'"'.$checked.' /></td>'."\n";
                if ($bg == '#eeeeee') {
                    $bg = '#dddddd';
                } else {
                    $bg = '#eeeeee';
                }
            }
            if ($this->precise) {
                $checked = (in_array('N/A', $data->{'q'.$this->id}) ? ' checked="checked"' : '');
                echo '<td width="40" align="center" bgcolor="'.$bg.'">';
                echo '<input name="'.$str.'" type="radio" value="N/A"'.$checked.' /></td>'."\n";
            }
            echo '    </tr>'."\n";
        }
        echo '  </tbody>'."\n";
        echo '</table>'."\n";
    }

    function date_survey_display($data) {
        global $ESPCONFIG;

        $date_mess = '(e.g. '.QUESTIONNAIREDATEFORMAT.')';
        $date_mess = str_replace('%m', '4', $date_mess);
        $date_mess = str_replace('%d', '21', $date_mess);
        $date_mess = str_replace('%y', '02', $date_mess);
        $date_mess = str_replace('%Y', '2002', $date_mess);

        if (isset($data->{'q'.$this->id})) {
            $dateparts = split('-', $data->{'q'.$this->id});
            $data->{'q'.$this->id} =
                strftime(QUESTIONNAIREDATEFORMAT,
                         mktime(0, 0, 0, $dateparts[1], $dateparts[2], $dateparts[0]));
        }
        echo '<input type="text" size="10" name="q'.$this->id.'" maxlength="10" value="'.
             (isset($data->{'q'.$this->id})?$data->{'q'.$this->id}:'').'" />';
        echo ' <em>'.$date_mess.'</em>';
    }

    function numeric_survey_display($data) {
        $this->length++; // for sign
        if($this->precise) {
            $this->length += 1 + $this->precise;
        }
        echo '<input type="text" size="'.$this->length.'" name="q'.$this->id.'" maxlength="'.$this->length.
             '" value="'.(isset($data->{'q'.$this->id}) ? $data->{'q'.$this->id} : '').'" />';
    }

    function sectiontext_survey_display($data) {
        echo '
    <div class="questioncontainer">
        <div class="qnOuter">
            <table class="qnInnerTable" width="100%" cellpadding="10"  cellspacing="1">
                <tr>
                    <td class="qnInnerTd" valign="top">&nbsp;</td>
                    <td class="qnInner" height="35">' .
                    format_text($this->content, FORMAT_HTML).'
                    </td>
                </tr>
            </table>
        </div>
    </div>
        ';
    }

///***
    function response_display($data, $qnum='') {
        global $QTYPENAMES;

        $method = $QTYPENAMES[$this->type_id].'_response_display';
        if (method_exists($this, $method)) {
            $this->questionstart_survey_display($qnum);
            $this->$method($data);
            $this->questionend_survey_display($qnum);
        } else {
            error('Display method not defined for question.');
        }
    }

    function yesno_response_display($data) {
        static $stryes = null;
        static $strno = null;
        static $uniquetag = 0;  // To make sure all radios have unique names.

        if (is_null($stryes)) {
             $stryes = get_string('yes');
             $strno = get_string('no');
        }

        $val1 = 'Y';
        $val2 = 'N';

        echo '<div class="response yesno">';
        if ($data->{'q'.$this->id} == $val1) {
        	echo '<span class="selected">' .
                 '<input type="radio" name="q'.$this->id.$uniquetag++.'y" checked="1" /> '.$stryes.'</span>';
        } else {
        	echo '<span class="unselected">' .
                 '<input type="radio" name="q'.$this->id.$uniquetag++.'y" onClick="this.checked=false;" /> '.$stryes.'</span>';
        }
        if ($data->{'q'.$this->id} == $val2) {
            echo ' <span class="selected">' .
                 '<input type="radio" name="q'.$this->id.$uniquetag++.'n" checked="1" /> '.$strno.'</span>';
        } else {
            echo ' <span class="unselected">' .
                 '<input type="radio" name="q'.$this->id.$uniquetag++.'n" onClick="this.checked=false;" /> '.$strno.'</span>';
        }
        echo '</div>';
    }

    function text_response_display($data) {
        echo '<div class="response text">';
        print_simple_box('<span class="selected">'.$data->{'q'.$this->id},'</span>');
        echo '</div>';
    }

    function essay_response_display($data) {
        echo '<div class="response text">';
        print_simple_box('<span class="selected">'.
                         (!empty($data->{'q'.$this->id}) ? $data->{'q'.$this->id} : '&nbsp;').
                         '</span>');
        echo '</div>';
    }

    function radio_response_display($data) {
        static $uniquetag = 0;  // To make sure all radios have unique names.

        $idcounter = 0;
        $currentradio = 0;
        $checked = (isset($data->{'q'.$this->id})?$data->{'q'.$this->id}:'');
        echo '<div class="response radio">';
        foreach ($this->choices as $id => $choice) {
            if (strpos($choice->content, '!other') !== 0) {
                if ($id == $checked) {
                    echo '<span class="selected">'.
                         '<input type="radio" name="'.$id.$uniquetag++.'" checked="1" /> '.
                         ($choice->content === '' ? $id : $choice->content).'</span><br />';
                } else {
                    echo '<span class="unselected">'.
                         '<input type="radio" name="'.$id.$uniquetag++.'" onClick="this.checked=false;" /> '.
                         ($choice->content === '' ? $id : $choice->content).'</span><br />';
                }
                $currentradio = ($currentradio + 1) % 2;

            } else {
                $other_text = preg_replace(
                        array("/^!other=/","/^!other/"),
                        array('','Other:'),
                        $choice->content);
                $cid = 'q'.$this->id.'_'.$id;

                if (isset($data->{'q'.$this->id.'_'.$id})) {
                    echo '<span class="selected">'.
                         '<input type="radio" name="'.$id.$uniquetag++.'" checked="1" /> '.$other_text.' ';
                    echo '<span class="response text">';
                    echo (!empty($data->$cid) ? htmlspecialchars($data->$cid) : '&nbsp;');
                    echo '</span></span><br />';
                } else {
                    echo '<span class="unselected"><input type="radio" name="'.$id.$uniquetag++.'" onClick="this.checked=false;" /> '.
                         $other_text.'</span><br />';
                }
            }
        }
        echo '</div>';
    }

    function check_response_display($data) {
        static $uniquetag = 0;  // To make sure all radios have unique names.

        if (!isset($data->{'q'.$this->id}) || !is_array($data->{'q'.$this->id})) {
            $data->{'q'.$this->id} = array();
        }

        echo '<div class="response check">';
        foreach ($this->choices as $id => $choice) {
            if (strpos($choice->content, '!other') !== 0) {
                if (in_array($id, $data->{'q'.$this->id})) {
                    echo '<span class="selected">'.
                         '<input type="checkbox" name="'.$id.$uniquetag++.'" checked="1" onClick="this.checked=true;" /> '.
                         ($choice->content === '' ? $id : $choice->content).'</span><br />';
                } else {
                    echo '<span class="unselected">'.
                         '<input type="checkbox" name="'.$id.$uniquetag++.'" onClick="this.checked=false;" /> '.
                         ($choice->content === '' ? $id : $choice->content).'</span><br />';
                }
            } else {
                $other_text = preg_replace(
                        array("/^!other=/","/^!other/U"),
                        array('','Other:'),
                        $choice->content);
                $cid = 'q'.$this->id.'_'.$id;

                if (isset($data->$cid)) {
                    echo '<span class="selected">'.
                         '<input type="checkbox" name="'.$id.$uniquetag++.'" checked="1" onClick="this.checked=true;" /> '.
                         ($other_text === '' ? $id : $other_text).' ';
                    echo '<span class="response text">';
                    echo (!empty($data->$cid) ? htmlspecialchars($data->$cid) : '&nbsp;');
                    echo '</span></span><br />';
                } else {
                    echo '<span class="unselected">'.
                         '<input type="checkbox" name="'.$id.$uniquetag++.'" onClick="this.checked=false;" /> '.
                         ($other_text === '' ? $id : $other_text).'</span><br />';
                }
            }
        }
        echo '</div>';
    }

    function drop_response_display($data) {
        static $uniquetag = 0;  // To make sure all radios have unique names.

        $options = array();
        foreach ($this->choices as $id => $choice) {
            $options[$id] = $choice->content;
        }
        echo '<div class="response drop">';
        choose_from_menu ($options, 'q'.$this->id.$uniquetag++, (isset($data->{'q'.$this->id})?$data->{'q'.$this->id}:''));
        echo ': <span class="selected">'.$options[$data->{'q'.$this->id}].'</span></div>';

    }

    function rate_response_display($data) {
        static $uniquetag = 0;  // To make sure all radios have unique names.
    
        if (!isset($data->{'q'.$this->id}) || !is_array($data->{'q'.$this->id})) {
            $data->{'q'.$this->id} = array();
        }

        echo '<div class="response rate">';

        echo '<table border="0" cellspacing="1" cellpadding="0">'."\n";
        echo '  <tbody>'."\n";
        echo '    <tr>'."\n";
        echo '      <td></td>'."\n";
        $bg = '#eeeeee';
        for ($j = 0; $j < $this->length; $j++) {
            echo '      <td width="40" align="center" bgcolor="'.$bg.'">'.($j+1).'</td>'."\n";
            if ($bg == '#eeeeee') {
                $bg = '#dddddd';
            } else {
                $bg = '#eeeeee';
            }
        }
        if ($this->precise) {
            echo '      <td width="40" align="center" bgcolor="'.$bg.'">N/A</td>'."\n";
        }
        echo '    </tr>'."\n";
    
        foreach ($this->choices as $cid => $choice) {
            $str = 'q'."{$this->id}_$cid";
            echo '    <tr>'."\n";
            echo '      <td align="right">'.$choice->content.'</td>'."\n";
            $bg = '#eeeeee';
            for ($j = 0; $j < $this->length; $j++) {
                $checked = ((isset($data->$str) && ($j == $data->$str)) ? ' checked="checked"' : '');
                echo '      <td width="40" align="center" bgcolor="'.$bg.'">';
                if ($checked) {
                    echo '<span class="selected">'.
                         '<input type="radio" name="'.$str.$j.$uniquetag++.'" checked="1" /></span>';
                } else {
                	echo '<span class="unselected">'.
                         '<input type="radio" name="'.$str.$j.$uniquetag++.'" onClick="this.checked=false;" /></span>';
                }
                if ($bg == '#eeeeee') {
                    $bg = '#dddddd';
                } else {
                    $bg = '#eeeeee';
                }
            }
            if ($this->precise) {
                $checked = (in_array('N/A', $data->{'q'.$this->id}) ? ' checked="checked"' : '');
                echo '<td width="40" align="center" bgcolor="'.$bg.'">';
                if ($checked) {
                    echo '<span class="selected">'.
                         '<input type="radio" name="'.$str.$uniquetag++.'na" checked="1" /></span>';
                } else {
                    echo '<span class="unselected">'.
                         '<input type="radio" name="'.$str.$uniquetag++.'na" onClick="this.checked=false;" /></span>';
                }
            }
            echo '    </tr>'."\n";
        }
        echo '  </tbody>'."\n";
        echo '</table>'."\n";
        echo '</div>';
    }

    function date_response_display($data) {
        global $ESPCONFIG;

        $date_mess = '(e.g. '.QUESTIONNAIREDATEFORMAT.')';
        $date_mess = str_replace('%m', '4', $date_mess);
        $date_mess = str_replace('%d', '21', $date_mess);
        $date_mess = str_replace('%y', '02', $date_mess);
        $date_mess = str_replace('%Y', '2002', $date_mess);

        if (isset($data->{'q'.$this->id})) {
            $dateparts = split('-', $data->{'q'.$this->id});
            $data->{'q'.$this->id} =
                strftime(QUESTIONNAIREDATEFORMAT,
                         mktime(0, 0, 0, $dateparts[1], $dateparts[2], $dateparts[0]));
        }
        echo '<div class="response date">';
        print_simple_box('<span class="selected">'.$data->{'q'.$this->id}.'</span>');
        echo '</div>';
    }

    function numeric_response_display($data) {
        $this->length++; // for sign
        if($this->precise) {
            $this->length += 1 + $this->precise;
        }
        echo '<div class="response numeric">';
        print_simple_box('<span class="selected">'.$data->{'q'.$this->id}.'</span>', '', '*');
        echo '</div>';
    }

    function sectiontext_response_display($data) {
        echo '
    <div class="questioncontainer">
        <div class="qnOuter">
            <table class="qnInnerTable" width="100%" cellpadding="10"  cellspacing="1">
                <tr>
                    <td class="qnInnerTd" valign="top">&nbsp;</td>
                    <td class="qnInner" height="35">' .
                    $this->content.'
                    </td>
                </tr>
            </table>
        </div>
    </div>
        ';
    }
///****

    /* {{{ proto void mkrespercent(array weights, int total, int precision, bool show_totals)
      Builds HTML showing PERCENTAGE results. */
    function mkrespercent($total, $precision, $showTotals) {
        global $CFG;
    
        $i=0;
        $bg='';
        $image_url = $CFG->wwwroot.'/mod/questionnaire/images/';

        $strtotal = get_string('total', 'questionnaire');
        $table = new Object();
        $table->size = array('*', '40%', '10%');
        $table->align = array('left', 'left', 'right');
        $table->head = array(get_string('response', 'questionnaire'),
                             get_string('average', 'questionnaire'), get_string('total', 'questionnaire'));

        if (!empty($this->counts) && is_array($this->counts)) {
            while(list($content,$num) = each($this->counts)) {
                if($num>0) { $percent = $num/$total*100.0; }
                else { $percent = 0; }
                if($percent > 100) {
                	$percent = 100;
                }
                if ($num) {
                    $out = '&nbsp;<img src="'.$image_url.'hbar_l.gif" height="9" width="4" />'.
                           sprintf('<img src="'.$image_url.'hbar.gif" height="9" width="%d" />', $percent*2).
                           '<img src="'.$image_url.'hbar_r.gif" height="9" width="4" />'.
                           sprintf('&nbsp;%.'.$precision.'f%%', $percent);
                } else {
                    $out = '';
                }
                $table->data[] = array($content, $out, $num);
                $i += $num;
            } // end while
    
            if($showTotals) {
                if($i>0) { $percent = $i/$total*100.0; }
                else { $percent = 0; }
                if($percent > 100) {
                	$percent = 100;
                }
    
                $out = '&nbsp;<img src="'.$image_url.'hbar_l.gif" height="9" width="4" />'.
                       sprintf('<img src="'.$image_url.'hbar.gif" height="9" width="%d" />', $percent*2).
                       '<img src="'.$image_url.'hbar_r.gif" height="9" width="4" />'.
                       sprintf('&nbsp;%.'.$precision.'f%%', $percent);
                $table->data[] = 'hr';
                $table->data[] = array($strtotal, $out, "$i / $total");
            }
        } else {
            $table->data[] = array('', get_string('noresponsedata', 'questionnaire'));
        }

        print_table($table);
    }

    /* {{{ proto void mkreslist(array weights, int total, int precision, bool show_totals)
        Builds HTML showing LIST results. */
    function mkreslist($total, $precision, $showTotals) {
        global $CFG;
    
        if($total == 0) {
        	return;
        }

        $strresponse = get_string('response', 'questionnaire');
        $strnum = get_string('num', 'questionnaire');
        $table = new Object();
        $table->align = array('left', 'left');
    
        $image_url = $CFG->wwwroot.'/mod/questionnaire/images/';

        $table->head = array($strnum, $strresponse);

        if (!empty($this->counts) && is_array($this->counts)) {
            while(list($text,$num) = each($this->counts)) {
            	$table->data[] = array($num, $text);
            }
        } else {
        	$table->data[] = array('', get_string('noresponsedata', 'questionnaire'));
        }

        print_table($table);
    }

    /* {{{ proto void mkresavg(array weights, int total, int precision, bool show_totals)
        Builds HTML showing AVG results. */
    function mkresavg($total, $precision, $showTotals, $length) {
        global $CFG;
    
        $stravg = '<div align="center">'.get_string('averagerank', 'questionnaire').'</div>';
        $table = new Object();
        $table->align = array('left', 'left', 'right');
        $table->head = array('', $stravg, '');
        $table->size = array('*', '40%', '10%');

        $image_url = $CFG->wwwroot.'/mod/questionnaire/images/';

        if (!$length) {
            $length = 5;
        }
        $width = 100 / $length;

        $out = '<table width="100%" cellpadding="0" cellspacing="0" border="0"><tr>';
        for ($i = 1; $i <= $length; $i++) {
        	$out .= '<td width="'.$width.'%" align="right">'.$i.'</td>';
        }
        $out .= '</tr></table>';
        $table->data[] = array('', $out, '');

        if (!empty($this->counts) && is_array($this->counts)) {
            while(list($content,$avg) = each($this->counts)) {
                if($avg) {
                	$out = '<img src="'.$image_url.'hbar_l.gif" height="9" width="4" />';
                    if (($j = $avg * $width) > 0) {
                        $out .= sprintf('<img src="'.$image_url.'hbar.gif" height="9" width="%d%%" />', $j-3);
                    }
                    $out .= '<img src="'.$image_url.'hbar_r.gif" height="9" width="4">';
                }
                $table->data[] = array($content, $out, sprintf('%.'.$precision.'f', $avg));
            } // end while
        } else {
            $table->data[] = array('', get_string('noresponsedata', 'questionnaire'));
        }

        print_table($table);
    }

    /* {{{ proto void mkresrank(array weights, int total, int precision, bool show_totals)
       Builds HTML showing RANK results. */
    function mkresrank($total, $precision, $showTotals) {
        global $CFG;
    
        $bg='';
        $image_url = $CFG->wwwroot.'/mod/questionnaire/images/';
    ?>
    <table border="0">
        <tr>
            <td align="right"><b><?php print_string('rank', 'questionnaire'); ?></b></td>
            <td>&nbsp;</td>
            <td>&nbsp;</td>
            <td>&nbsp;</td>
        </tr>
    <?php
        arsort($this->counts);
        $i=0; $pt=0;
        while(list($content,$num) = each($counts)) {
            if($num)
                $p = $num/$total*100.0;
            else
                $p = 0;
            $pt += $p;
    
            if($bg != $GLOBALS['ESPCONFIG']['bgalt_color1'])
                $bg = $GLOBALS['ESPCONFIG']['bgalt_color1'];
            else
                $bg = $GLOBALS['ESPCONFIG']['bgalt_color2'];
    ?>
        <tr bgcolor="<?php echo($bg); ?>">
            <td align="right"><b><?php echo(++$i); ?></b></td>
            <td><?php echo($content); ?></td>
            <td align="right" width="60"><?php if($p) printf("%.${precision}f%%",$p); ?></td>
            <td align="right" width="60">(<?php echo($num); ?>)</td>
        </tr>
    <?php
        } // end while
        if($showTotals) {
            if($bg != $GLOBALS['ESPCONFIG']['bgalt_color1'])
                $bg = $GLOBALS['ESPCONFIG']['bgalt_color1'];
            else
                $bg = $GLOBALS['ESPCONFIG']['bgalt_color2'];
    ?>
        <tr bgcolor="<?php echo($bg); ?>">
            <td colspan=2 align="left"><b><?php print_string('total', 'questionnaire'); ?></b></td>
            <td align="right"><b><?php printf("%.${precision}f%%",$pt); ?></b></td>
            <td align="right"><b><?php echo($total); ?></b></td>
        </tr>
    <?php } ?>
    </table>
    <?php
    }
}
?>
