<?PHP  // $Id: lib.php,v 1.13 2006/02/04 16:52:00 gustav_delius Exp $

/// Library of functions and constants for module scheduler

define('SCHEDULER_TIMEUNKNOWN', 0);  // This is used for appointments for which no time is entered

function scheduler_add_instance($scheduler) {
/// Given an object containing all the necessary data, 
/// (defined by the form in mod.html) this function 
/// will create a new instance and return the id number 
/// of the new instance.
    global $SESSION,$CFG;

    $scheduler->timemodified = time();
    
    $id = insert_record("scheduler", $scheduler);
    
    return $id;
}


function scheduler_update_instance($scheduler) {
/// Given an object containing all the necessary data, 
/// (defined by the form in mod.html) this function 
/// will update an existing instance with new data.

    $scheduler->timemodified = time();
    $scheduler->id = $scheduler->instance;

    # May have to add extra stuff in here #

    return update_record("scheduler", $scheduler);
}


function scheduler_delete_instance($id) {
/// Given an ID of an instance of this module, 
/// this function will permanently delete the instance 
/// and any data that depends on it.  

    if (! $scheduler = get_record("scheduler", "id", "$id")) {
        return false;
    }

    $result = true;

    # Delete any dependent records here #

    if (! delete_records("scheduler", "id", "$scheduler->id")) {
        $result = false;
    }
    if (!delete_records('scheduler_slots', 'scheduler', $scheduler->id)) {
        $result = false;
    }
    
    // delete all associated events from calendar
    if ($events = get_records_select('event', "modulename = 'scheduler' and instance = '$id'")) {
        foreach($events as $event) {
            delete_event($event->id);
        }
    }

    return $result;
}

function scheduler_user_outline($course, $user, $mod, $scheduler) {
/// Return a small object with summary information about what a 
/// user has done with a given particular instance of this module
/// Used for user activity reports.
/// $return->time = the time they did it
/// $return->info = a short text description

    return $return;
}

function scheduler_user_complete($course, $user, $mod, $scheduler) {
/// Print a detailed representation of what a  user has done with 
/// a given particular instance of this module, for user activity reports.

    return true;
}

function scheduler_print_recent_activity($course, $isteacher, $timestart) {
/// Given a course and a time, this module should find recent activity 
/// that has occurred in scheduler activities and print it out. 
/// Return true if there was output, or false is there was none.

    global $CFG;

    return false;  //  True if anything was printed, otherwise false 
}

function scheduler_cron () {
/// Function to be run periodically according to the moodle cron
/// This function searches for things that need to be done, such 
/// as sending out mail, toggling flags etc ... 

    global $CFG;
    $date = make_timestamp(date("Y"), date("m"), date("d"), date("H"), date("i"));
    // for every appointment
//    if ($slots = get_records('scheduler_slots', 'emaildate', ">0")) {
      if ($slots = get_records_sql("SELECT * FROM ". $CFG->prefix . "scheduler_slots WHERE emaildate > 0 AND student > 0 AND emaildate <=$date ORDER BY starttime ")){
        foreach ($slots as $slot) {
            //if no email previously sent and one is required
            if (($slot->emaildate <= $date) & ($slot->student > 0)) {
                $teacher = get_record('user', 'id', $slot->teacher);
                if(email_to_user(get_record('user', 'id', $slot->student),$teacher,"Scheduled Appointment with ".fullname($teacher),"Scheduled Appointment with ".fullname($teacher)." on ".date("l jS F Y",$slot->starttime)." from ".date("H:i",$slot->starttime)." until ".date("H:i",$slot->starttime + $slot->duration).".  The location for this appointment is ".$slot->location.".")){
                    $slot->emaildate  = -1;
                    update_record('scheduler_slots', $slot);
                }
            }
        }
    }
    return true;
}

function scheduler_grades($schedulerid) {
/// Must return an array of grades for a given instance of this module, 
/// indexed by user.  It also returns a maximum allowed grade.

    return NULL;
}

function scheduler_get_participants($schedulerid) {
//Returns the users with data in one scheduler
//(users with records in journal_entries, students and teachers)

    global $CFG;

    //Get students
    $students = get_records_sql("SELECT DISTINCT u.*
                                 FROM {$CFG->prefix}user u,
                                      {$CFG->prefix}scheduler_slots s
                                 WHERE s.scheduler = '$schedulerid' and
                                       u.id = s.student");
    //Get teachers
    $teachers = get_records_sql("SELECT DISTINCT u.*
                                 FROM {$CFG->prefix}user u,
                                      {$CFG->prefix}scheduler s
                                 WHERE s.id = '$schedulerid' and
                                       u.id = s.teacher");

    //Add teachers to students
    if ($teachers) {
        foreach ($teachers as $teacher) {
            $students[$teacher->id] = $teacher;
        }
    }
    //Return students array (it contains an array of unique users)
    return ($students);
}

//////////////////////////////////////////////////////////////////////////////////////
/// Any other scheduler functions go here.  Each of them must have a name that 
/// starts with scheduler_

function scheduler_userdate($date,$local=0) {
/// Parameter $local added by power-web.at
/// When local Date is needed the $local Param must be set to 1 
    if ($date == 0) {
        return '';
    } else {
        return userdate($date, get_string('strftimedaydate'));
    }
}

function scheduler_usertime($date,$local=0) {
/// Parameter $local added by power-web.at 
/// When local Time is needed the $local Param must be set to 1 
    if ($date == 0) {
            return '';
    } else {
        if (!$timeformat = get_user_preferences('calendar_timeformat')) {
            $timeformat = get_string('strftimetime');
        }
        return userdate($date, $timeformat);    
    }
}

function scheduler_conflict($schedulerid, $starttime, $endtime, $teacher) {
// Returns an array of slots that would overlap with this.
    $result = false;
    if ($slots = get_records('scheduler_slots', 'scheduler', $schedulerid)) {
        foreach ($slots as $slot) {
            if (($slot->starttime + $slot->duration*60 > $starttime) and ($slot->starttime < $endtime) and ($slot->exclusive == 1) and ($teacher == $slot->teacher)) {
                $result[] = $slot;
            }
        }
    }
    return $result;
}

function scheduler_conflict_exclude_thisone($schedulerid, $starttime, $endtime, $schedulerid, $teacher) {
// Returns true if there already exists a slot that would overlap with this.
    if ($slots = get_records('scheduler_slots', 'scheduler', $schedulerid)) {
        foreach ($slots as $slot) {
            if (($slot->starttime + $slot->duration*60 > $starttime) and ($slot->starttime < $endtime) and ($slot->exclusive == 1) and ($schedulid != $slot->id) and ($teacher == $slot->teacher)) {
                return true;
            }
        }
    }
    return false;
}


function scheduler_noneexclusive($schedulerid, $starttime, $endtime) {
// Returns true if there already exists a slot that would overlap with this.
    if ($slots = get_records('scheduler_slots', 'scheduler', $schedulerid)) {
        foreach ($slots as $slot) {
            if (($slot->starttime + $slot->duration*60 > $starttime) and ($slot->starttime < $endtime) and ($slot->exclusive == 0)) {
                return 0;
            }
            if (($slot->starttime + $slot->duration*60 > $starttime) and ($slot->starttime < $endtime) and ($slot->exclusive == 0)) {
                return 1;
            }
        }
    }
    return 1;
}

function scheduler_add_update_calendar_events($slot, $course) {
/// Updates events in the calendar to the information provided.
/// If the events do not yet exist it creates them.
/// The only argument this function requires is the complete database record of a scheduler slot.
/// The course parameter should be the full record of the course for this scheduler so the teacher-title and student-title can
///-be determined.
    
    //firstly, collect up the information we'll need no matter what.
    $eventDuration = ($slot->duration)*60;
    $eventStartTime = $slot->starttime;
    $eventStudent = $slot->student;
    $teacher = get_record('user', 'id', $slot->teacher);
    $student = get_record("user","id",$slot->student);
    $teacherEventName = get_string("meetingwith","scheduler")." ".$course->student.", ".fullname($student);
    $studentEventName = get_string("meetingwith","scheduler")." ".$course->teacher.", ".fullname($teacher);
    
    $schedulerDescription = get_field("scheduler","description","id",$slot->scheduler);
    $schedulerName = get_field("scheduler","name","id",$slot->scheduler);
    $teacherEventDescription = "$schedulerName<br><br>$schedulerDescription";
                                  
    $studentEventDescription = $teacherEventDescription;
    
    $courseID = get_field("scheduler","course","id",$slot->scheduler);
    
    //the eventtype field stores a code that is used to relate calendar events with the slots that 'own' them.
    //the code is SSstu (for a student event) or SSsup (for a teacher event).
    //then, the id of the scheduler slot that it belongs to.
    //finally, the courseID. I can't remember why, TODO: remember the good reason.
    //all in a colon delimited string. This will run into problems when the IDs of slots and courses are bigger than 7 digits in length...    
    $teacherEventType = "SSsup:".$slot->id.":".$courseID;
    $studentEventType = "SSstu:".$slot->id.":".$courseID;
    
    scheduler_events_exists($slot, $studentEvent, $teacherEvent);
    //firstly, deal with the student's event
    //if it exists, update it, else create a new one.
    
    if ($studentEvent) {
        $studentEvent->name = $studentEventName;
        $studentEvent->description = $studentEventDescription;
        $studentEvent->format = 1;
        $studentEvent->userid = $eventStudent;
        $studentEvent->modulename = 'scheduler';
        $studentEvent->instance = $slot->scheduler;
        $studentEvent->timestart = $eventStartTime;
        $studentEvent->timeduration = $eventDuration;
        $studentEvent->visible = 1;
        $studentEvent->eventtype = $studentEventType;
        update_record("event",$studentEvent);
    } else {
        $studentEvent->name = $studentEventName;
        $studentEvent->description = $studentEventDescription;
        $studentEvent->format = 1;
        $studentEvent->userid = $eventStudent;
        $studentEvent->modulename = 'scheduler';
        $studentEvent->instance = $slot->scheduler;
        $studentEvent->timestart = $eventStartTime;
        $studentEvent->timeduration = $eventDuration;
        $studentEvent->visible = 1;
        $studentEvent->id = null;
        $studentEvent->eventtype = $studentEventType;
        // This should be changed to use add_event()
        insert_record("event", $studentEvent);
    }
    
    if ($teacherEvent) {
        $teacherEvent->name = $teacherEventName;
        $teacherEvent->description = $teacherEventDescription;
        $teacherEvent->format = 1;
        $teacherEvent->userid = $slot->teacher;
        $teacherEvent->modulename = 'scheduler';
        $teacherEvent->timestart = $eventStartTime;
        $teacherEvent->timeduration = $eventDuration;
        $teacherEvent->visible = 1;
        $teacherEvent->eventtype = $teacherEventType;
        update_record("event",$teacherEvent);
    } else {
        $teacherEvent->name = $teacherEventName;
        $teacherEvent->description = $teacherEventDescription;
        $teacherEvent->format = 1;
        $teacherEvent->userid = $slot->teacher;
        $teacherEvent->modulename = 'scheduler';
        $teacherEvent->timestart = $eventStartTime;
        $teacherEvent->timeduration = $eventDuration;
        $teacherEvent->visible = 1;
        $teacherEvent->id = null;
        $teacherEvent->eventtype = $teacherEventType;
        insert_record("event", $teacherEvent);
    }
}


function scheduler_delete_calendar_events($slot) {
///will delete calendar events for a given scheduler slot, and not complain if the record does not exist.
/// The only argument this function requires is the complete database record of a scheduler slot.
    

    $courseID = get_field("scheduler","course","id",$slot->scheduler);
    
    $teacherEventType = "SSsup:".$slot->id.":".$courseID;
    $studentEventType = "SSstu:".$slot->id.":".$courseID;
    
    $teacherDeletionSuccess = delete_records("event","eventtype",$teacherEventType);
    $studentDeletionSuccess = delete_records("event","eventtype",$studentEventType);
    
    return ($teacherDeletionSuccess && $studentDeletionSuccess);
    //this return may not be meaningful if the delete records functions do not return anything meaningful.
}

function scheduler_events_update($slot, $course) {
///This function decides if a slot should have calendar events associated with it, and calls the update/delete functions if neccessary.
///it must be passed the complete scheduler_slots record to function correctly.
///The course parameter should be the record that belongs to the course for this scheduler.
   
    $slotDoesntHaveAStudent = !($slot->student);
    $slotWasAttended = $slot->attended;   
    $slotHasNoTime = ($slot->starttime == SCHEDULER_TIMEUNKNOWN);
     

    if ($slotDoesntHaveAStudent || $slotWasAttended || $slotHasNoTime) {
        scheduler_delete_calendar_events($slot);
    } else {
        scheduler_add_update_calendar_events($slot, $course);
    }

}

function scheduler_events_exists($slot, &$studentSlot, &$teacherSlot) {
///This function sets the $studentSlot and $teacherSlot to the records of the calendar that relate to these scheduler slots.
///They will equal false if the records do not exist.
///it requires the full record of the scheduler slot supplied as $slot.
    
//first we need to know the course that the scheduler belongs to...
    $courseID = get_field("scheduler","course","id",$slot->scheduler);
    
//now try to fetch the event records...
    $teacherEventType = "SSsup:".$slot->id.":".$courseID;
    $studentEventType = "SSstu:".$slot->id.":".$courseID;
    
    $teacherSlot = get_record("event","eventtype",$teacherEventType);
    $studentSlot = get_record("event","eventtype",$studentEventType);
    
}


?>
